@@ -1,10 +1,7 @@
-package # hide from PAUSE
+package # hide from PAUSE 
     DBICTest::Schema::TypedObject;
 
-use warnings;
-use strict;
-
-use base qw/DBICTest::BaseResult/;
+use base qw/DBIx::Class::Core/;
 
 __PACKAGE__->table('typed_object');
 __PACKAGE__->add_columns(
